# -*- coding: utf-8 -*-

import sys
import os
import logging
import datetime
import re
import threading

from PyQt5 import QtCore, QtGui, QtWidgets, uic

from lib.hardware import Hardware
from lib.su import Su
from lib.disk import Disk

from lib.gr.file_sink import FileSink

class SuGui(QtWidgets.QMainWindow):
    """
    GUI class used to realise the main window.
    """
    def __init__(self):
        """
        Instantiate GUI class and open window.
        GUI elements are directly loaded and converted from UI file.
        """
        super(SuGui, self).__init__()
        uic.loadUi("lib/gui/su_mainwindow.ui", self)
        self.show()
        # setting some defaults
        self.setWindowTitle(Su.name_and_version())
        self._set_default_dump_file_path()
        self.lineEdit_dump_file.setCursorPosition(0)
        self.refresh_hw_button()
        self.button_stop.setEnabled(False)
        # button events
        self.button_start.clicked.connect(self.start_button)
        self.button_stop.clicked.connect(self.stop_button)
        self.button_close.clicked.connect(self.close_button)
        self.button_hw_refresh.clicked.connect(self.refresh_hw_button)
        # menubar
        self.actionQuit.triggered.connect(self.close_button)
        self.actionAbout.triggered.connect(self.about_button)
        # edit lines
        self.lineEdit_dump_file.editingFinished. \
            connect(self.check_dump_file_path)
        # radio buttons
        self.radio_buttons_samp_rate_and_time = {
            # samp_rate
            self.rb_1mhz: 1e6,
            self.rb_2mhz: 2e6,
            self.rb_3mhz: 3e6,
            # time to measure in seconds
            self.rb_1m: 60,
            self.rb_5m: 300,
            self.rb_10m: 600,
            self.rb_1h: 3600,
            self.rb_24h: 86400
        }
        self.radio_buttons_freq = {
            self.rb_433: 433e6,
            self.rb_868: 868e6
        }
        # add released actions to radio buttons
        for button in self.radio_buttons_samp_rate_and_time.keys():
            button.released.connect(self._update_status_label)

        # labels
        self._update_status_label()
        self.label_file_overwrite.setText("change path")
        # progress_bar
        self.progressBar.setValue(0)

    def start_button(self):
        """
        Start button action in main window.
        """
        if self._check_dependencies_before_start_capture():
            options = self._extract_options()
            file_sink = FileSink(
                options['dump_file_path'],
                options['freq'],
                options['samp_rate'],
                options['capture_time'],
                options['device']
            )
            self.button_stop.setEnabled(True)
            self.button_start.setEnabled(False)

            file_sink.capture()



    def stop_button(self):
        """
        Stop button action in main window.
        """
        pass

    def close_button(self):
        """
        Close button action in main window.
        """
        logging.info("Close application")
        QtCore.QCoreApplication.instance().quit()

    def refresh_hw_button(self):
        """
        Refresh combo box with current connected hardware.
        """
        self.comboBox_hardware.clear()
        for device in Hardware().get_connected_supported_devices():
            text = device['name'] + " (" + device['id'] + ")"
            self.comboBox_hardware.addItem(self.tr(text))

    def about_button(self):
        """
        Show about widget.
        """
        QtWidgets.QMessageBox.about(self, "About", \
                Su.name_and_version() + "\n\n" \
                "Contact: daniel.meissner@smail.inf.h-brs.de\n" \
                "Source: https://git.fslab.de/dmeiss2s/su")

    def check_dump_file_path(self):
        """
        Update labels for writable state and free disk space in path.
        """
        free_disk_space = Disk.free_disk_space(self.lineEdit_dump_file.text())
        if free_disk_space == None:
            self.label_disk_space.setText("")
            self.label_file_overwrite.setText("not writable")
            self.label_file_overwrite.setStyleSheet('color: red')
        else:
            self.label_file_overwrite.setText("")
            self.label_disk_space.setText("Free disk space:\n" \
                                          "" + Disk.sizeof_fmt(free_disk_space))
            if free_disk_space - self._calculate_needed_disk_space() < 0:
                self.label_disk_space.setStyleSheet('color: red')
            else:
                self.label_disk_space.setStyleSheet('color: black')


    def _get_default_dump_file_path(self):
        """
        Generate path for dump file. The default file path is defined as
        dump_file with current timestamp located in the home of current user.

        Example:
            /home/user/su/dump_file-1942_05_23-13_37_23.iq
        """
        dt = datetime.datetime.now().strftime("%Y_%m_%d-%H_%M_%S")
        path = os.path.expanduser("~") + "/su/dump_file-" + dt + ".iq"

        return path

    def _extract_selected_hardware_device(self):
        """
        Extract device key from combo box.
        """
        device_id = re.search(r"\w{4}:\w{4}", self.comboBox_hardware. \
                                                currentText()).group()

        device_key = ''
        for key, value in Hardware.SUPPORTED_HARDWARE.items():
            if value == device_id:
                device_key = key

        return device_key

    def _check_dependencies_before_start_capture(self):
        """
        Private method to check needed dependencies before starting capturing
        data. The following states needs to be passed:
            1. A dump file path need to be befined
            2. Supported RF hardware need to be connected
            3. RF hardware need to be selected

        It returns True or False.
        """
        if len(self.lineEdit_dump_file.text()) == 0:
            QtWidgets.QMessageBox.about(self, "Dumpfile path missing…", \
                                              "Please add dump file path.")
            return False
        elif len(Hardware.get_connected_supported_devices()) == 0:
            QtWidgets.QMessageBox.about(self, "No supported RF hardware "\
                                              "connected…", \
                                              "Please connect RF hardware.")
            self.refresh_hw_button()
            return False
        elif len(self.comboBox_hardware.currentText()) == 0:
            QtWidgets.QMessageBox.about(self, "No supported RF hardware "\
                                              "selected…", \
                                              "Select RF hardware.")
            self.refresh_hw_button()
            return False
        else:
            return True

    def _update_status_label(self):
        """
        Update status label.
        """
        needed_disk_space = Disk.sizeof_fmt(self._calculate_needed_disk_space())
        self.label_status. \
            setText("Needed disk space:\n" \
                    "" + needed_disk_space)
        self.check_dump_file_path()

    def _calculate_needed_disk_space(self):
        """
        Calculate needed disk space based on current check box status. Every
        sample needs 8 byte of disk space (32 bit I, 32 bit Q).

        Returned integer value.
        """
        time, samp_rate = self._get_checked_time_and_samp_rate_radio_buttons().values()

        return int((time*samp_rate) * 8)

    def _get_checked_time_and_samp_rate_radio_buttons(self):
        """
        Return a list of all checked radio buttons.
        """
        checked_buttons = {}
        for radio_button, value in \
                self.radio_buttons_samp_rate_and_time.iteritems():
            if radio_button.isChecked():
                checked_buttons[radio_button] = value

        return checked_buttons

    def _get_checked_freq_button(self):
        """
        Get checked freq button.
        """
        for radio_button, value in self.radio_buttons_freq.iteritems():
            if radio_button.isChecked():
                return {radio_button: value}

    def _set_default_dump_file_path(self):
        """
        Set default dump file path.
        """
        if "SU_DUMP_FILE_PATH" in os.environ:
            self.lineEdit_dump_file.setText(os.environ['SU_DUMP_FILE_PATH'])
        else:
            self.lineEdit_dump_file.setText(self._get_default_dump_file_path())

    def _extract_options(self):
        """
        Extract selected options from gui.
        """
        freq_button = self._get_checked_freq_button()
        time_and_samp_rate_buttons = self._get_checked_time_and_samp_rate_radio_buttons()
        # radio buttons
        options = {}
        options['capture_time'], options['samp_rate'] = \
            time_and_samp_rate_buttons.values()
        options['freq'] = freq_button.values()[0]
        # line inputs
        options['dump_file_path'] = self.lineEdit_dump_file.text()
        # dropdown boxes
        options['device'] = self._extract_selected_hardware_device()

        return options
